@@ -16,6 +16,12 @@
 #' the reference files will occur in a temporary directory, and will not be
 #' synchronised back to the source package.
 #'
+#' @section 3rd edition:
+#' `r lifecycle::badge("deprecated")`
+#'
+#' `expect_known_output()` and friends are deprecated in the 3rd edition;
+#' please use [expect_snapshot_output()] and friends instead.
+#'
 #' @export
 #' @param file File path where known value/output will be stored.
 #' @param update Should the file be updated? Defaults to `TRUE`, with
@@ -24,6 +30,7 @@
 #' @param version The serialization format version to use. The default, 2, was
 #'   the default format from R 1.4.0 to 3.5.3. Version 3 became the default from
 #'   R 3.6.0 and can only be read by R versions 3.5.0 and higher.
+#' @param ... Passed on to [waldo::compare()].
 #' @keywords internal
 #' @inheritParams expect_equal
 #' @inheritParams capture_output_lines
@@ -48,6 +55,11 @@ expect_known_output <- function(object, file,
                                 label = NULL,
                                 print = FALSE,
                                 width = 80) {
+
+  edition_deprecate(3, "expect_known_output()",
+    "Please use `expect_snapshot_output()` instead"
+  )
+
   act <- list()
   act$quo <- enquo(object)
   act$lab <- label %||% quo_label(act$quo)
@@ -76,12 +88,16 @@ compare_file <- function(path, lines, ..., update = TRUE, info = NULL) {
     warning("Reference output is not UTF-8 encoded", call. = FALSE)
   }
 
-  comp <- compare(lines, enc2native(old_lines), ...)
+  comp <- waldo_compare(
+    x = enc2native(old_lines), x_arg = "old",
+    y = lines, y_arg = "new",
+    ...
+  )
   expect(
-    comp$equal,
+    length(comp) == 0,
     sprintf(
-      "Results have changed from known value recorded in %s.\n%s",
-      encodeString(path, quote = "'"), comp$message
+      "Results have changed from known value recorded in %s.\n\n%s",
+      encodeString(path, quote = "'"), paste0(comp, collapse = "\n\n")
     ),
     info = info
   )
@@ -100,6 +116,10 @@ expect_known_value <- function(object, file,
                                info = NULL,
                                label = NULL,
                                version = 2) {
+  edition_deprecate(3, "expect_known_value()",
+    "Please use `expect_snapshot_value()` instead"
+  )
+
   act <- quasi_label(enquo(object), label, arg = "object")
 
   if (!file.exists(file)) {
@@ -113,7 +133,6 @@ expect_known_value <- function(object, file,
       saveRDS(act$val, file, version = version)
     }
 
-
     expect(
       comp$equal,
       sprintf(
@@ -131,6 +150,9 @@ expect_known_value <- function(object, file,
 #' @rdname expect_known_output
 #' @usage NULL
 expect_equal_to_reference <- function(..., update = FALSE) {
+  edition_deprecate(3, "expect_equal_to_reference()",
+    "Please use `expect_snapshot_value()` instead"
+  )
   expect_known_value(..., update = update)
 }
 
@@ -139,6 +161,10 @@ expect_equal_to_reference <- function(..., update = FALSE) {
 #' @param hash Known hash value. Leave empty and you'll be informed what
 #'   to use in the test output.
 expect_known_hash <- function(object, hash = NULL) {
+  edition_deprecate(3, "expect_known_hash()",
+    "Please use `expect_snapshot_value()` instead"
+  )
+
   act <- quasi_label(enquo(object), arg = "object")
   act_hash <- digest::digest(act$val)
   if (!is.null(hash)) {
