@@ -1,12 +1,5 @@
-#' Expectation: do C++ tests past?
-#'
-#' Test compiled code in the package `package`. A call to this function will
-#' automatically be generated for you in `tests/testthat/test-cpp.R` after
-#' calling [use_catch()]; you should not need to manually call this expectation
-#' yourself.
-#'
-#' @param package The name of the package to test.
 #' @keywords internal
+#' @rdname run_cpp_tests
 #' @export
 expect_cpp_tests_pass <- function(package) {
   run_testthat_tests <- get_routine(package, "run_testthat_tests")
@@ -15,7 +8,7 @@ expect_cpp_tests_pass <- function(package) {
   tests_passed <- TRUE
 
   tryCatch(
-    output <- capture_output_lines(tests_passed <- .Call(run_testthat_tests)),
+    output <- capture_output_lines(tests_passed <- .Call(run_testthat_tests, FALSE)),
     error = function(e) {
       warning(sprintf("failed to call test entrypoint '%s'", run_testthat_tests))
     }
@@ -27,11 +20,121 @@ expect_cpp_tests_pass <- function(package) {
   expect(tests_passed, paste("C++ unit tests:", info, sep = "\n"))
 }
 
+#' Do C++ tests past?
+#'
+#' Test compiled code in the package `package`. A call to this function will
+#' automatically be generated for you in `tests/testthat/test-cpp.R` after
+#' calling [use_catch()]; you should not need to manually call this expectation
+#' yourself.
+#'
+#' @param package The name of the package to test.
+#' @keywords internal
+#' @export
+run_cpp_tests <- function(package) {
+  check_installed("xml2", "run_cpp_tests()")
+
+  run_testthat_tests <- get_routine(package, "run_testthat_tests")
+
+  output <- ""
+  tests_passed <- TRUE
+
+  catch_error <- FALSE
+  tryCatch({
+    output <- capture_output_lines(tests_passed <- .Call(run_testthat_tests, TRUE))
+  },
+    error = function(e) {
+      catch_error <- TRUE
+      reporter <- get_reporter()
+
+      reporter$start_context(context = "Catch")
+      reporter$start_test(context = "Catch", test = "Catch")
+      reporter$add_result(context = "Catch", test = "Catch", result = expectation("failure", e$message))
+      reporter$end_test(context = "Catch", test = "Catch")
+      reporter$end_context(context = "Catch")
+    }
+  )
+
+
+  if (catch_error) {
+    return()
+  }
+
+  report <- xml2::read_xml(paste(output, collapse = "\n"))
+
+  contexts <- xml2::xml_find_all(report, "//TestCase")
+
+  for (context in contexts) {
+    context_name <- sub(" [|][^|]+$", "", xml2::xml_attr(context, "name"))
+
+    get_reporter()$start_context(context = context_name)
+
+    tests <- xml2::xml_find_all(context, "./Section")
+    for (test in tests) {
+      test_name <- xml2::xml_attr(test, "name")
+
+      result <- xml2::xml_find_first(test, "./OverallResults")
+      successes <- as.integer(xml2::xml_attr(result, "successes"))
+
+      get_reporter()$start_test(context = context_name, test = test_name)
+
+      for (i in seq_len(successes)) {
+        exp <- expectation("success", "")
+        exp$test <- test_name
+        get_reporter()$add_result(context = context_name, test = test_name, result = exp)
+      }
+
+      failures <- xml2::xml_find_all(test, "./Expression")
+      for (failure in failures) {
+        org <- xml2::xml_find_first(failure, "Original")
+        org_text <- xml2::xml_text(org, trim = TRUE)
+
+        filename <- xml2::xml_attr(failure, "filename")
+        type <- xml2::xml_attr(failure, "type")
+
+        type_msg <- switch(type,
+          "CATCH_CHECK_FALSE" = "isn't false.",
+          "CATCH_CHECK_THROWS" = "did not throw an exception.",
+          "CATCH_CHECK_THROWS_AS" = "threw an exception with unexpected type.",
+          "isn't true."
+        )
+
+        org_text <- paste(org_text, type_msg)
+
+        line <- xml2::xml_attr(failure, "line")
+        failure_srcref <- srcref(srcfile(file.path("src", filename)), c(line, line, 1, 1))
+
+        exp <- expectation("failure", org_text, srcref = failure_srcref)
+        exp$test <- test_name
+
+        get_reporter()$add_result(context = context_name, test = test_name, result = exp)
+      }
+
+      exceptions <- xml2::xml_find_all(test, "./Exception")
+      for (exception in exceptions) {
+        exception_text <- xml2::xml_text(exception, trim = TRUE)
+        filename <- xml2::xml_attr(exception, "filename")
+        line <- xml2::xml_attr(exception, "line")
+
+        exception_srcref <- srcref(srcfile(file.path("src", filename)), c(line, line, 1, 1))
+
+        exp <- expectation("error", exception_text, srcref = exception_srcref)
+        exp$test <- test_name
+
+        get_reporter()$add_result(context = context_name, test = test_name, result = exp)
+      }
+
+      get_reporter()$end_test(context = context_name, test = test_name)
+    }
+
+    get_reporter()$end_context(context = context_name)
+  }
+}
+
 #' Use Catch for C++ Unit Testing
 #'
 #' Add the necessary infrastructure to enable C++ unit testing
-#' in \R packages with
-#' \href{https://github.com/philsquared/Catch}{Catch} and `testthat`.
+#' in \R packages with [Catch](https://github.com/catchorg/Catch2) and
+#' `testthat`.
 #'
 #' Calling `use_catch()` will:
 #'
@@ -73,7 +176,7 @@ expect_cpp_tests_pass <- function(package) {
 #'
 #' All of the functions provided by Catch are
 #' available with the `CATCH_` prefix -- see
-#' \href{https://github.com/philsquared/Catch/blob/master/docs/assertions.md}{here}
+#' [here](https://github.com/catchorg/Catch2/blob/master/docs/assertions.md)
 #' for a full list. `testthat` provides the
 #' following wrappers, to conform with `testthat`'s
 #' \R interface:
@@ -148,8 +251,8 @@ expect_cpp_tests_pass <- function(package) {
 #' as necessary within your unit test suite.
 #'
 #' @export
-#' @seealso \href{https://github.com/philsquared/Catch}{Catch}, the
-#'   library used to enable C++ unit testing.
+#' @seealso [Catch](https://github.com/catchorg/Catch2/blob/master/docs/assertions.md),
+#'   the library used to enable C++ unit testing.
 use_catch <- function(dir = getwd()) {
   desc_path <- file.path(dir, "DESCRIPTION")
   if (!file.exists(desc_path)) {
@@ -251,5 +354,5 @@ get_routine <- function(package, routine) {
 }
 
 (function() {
-  .Call(run_testthat_tests)
+  .Call(run_testthat_tests, TRUE)
 })
